<!DOCTYPE KrossScripting SYSTEM "kpartgui.dtd">
<KrossScripting version="3">

    <collection name="view">
        <separator/>
        <script text="Statistics" comment="Statistics Script"
            name="statistics_script"
            interpreter="ruby"
            file="statistics.rb" />
    </collection>

</KrossScripting>
