<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="klinkstatus_shell" version="2">
<MenuBar>

  <Menu noMerge="0" name="file"><text>&amp;File</text>
    <Merge/>
    <Separator/>
    <Action name="file_quit"/>
  </Menu>

  <Menu noMerge="0" name="view"><text>&amp;View</text>
    <Separator/>
  </Menu>
            
  <Menu noMerge="0" name="search"><text>S&amp;earch</text>
      <Separator/>
  </Menu>

<!--  <Menu noMerge="1" name="validate"><text>&amp;Validate</text>
      <Separator/>
  </Menu>-->

  <Menu noMerge="0" name="tools"><text>&amp;Tools</text>
      <Separator/>
  </Menu>

  <Menu noMerge="0" name="window"><text>&amp;Window</text>
      <Separator/>
  </Menu>

</MenuBar>

</kpartgui>
