<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="klinkstatus_part" version="2">
    <MenuBar>
        
        <Menu name="file"><text>&amp;File</text>
            <Action name="new_link_check"/>
            <Action name="open_link"/>
            <Action name="close_tab"/>
            <Separator/>
            <Menu name="file_export_html"><text>Export to HTML</text>
                <Action name="file_export_html_all"/>
                <Action name="file_export_html_broken"/>
                <Separator/>
                <Action name="file_ghns"/>
            </Menu>
            <Action name="file_create_site_map"/>
            <Separator/>
        </Menu>
        
        <Menu name="settings"><text>&amp;Settings</text>
            <Action name="configure_klinkstatus"/>
        </Menu>
        
        <Menu name="view"><text>&amp;View</text>
            <Action name="hide_search_bar"/>
            <Action name="reset_search_bar"/>
            <Separator/>
            <Action name="follow_last_link_checked"/>
            <Action name="disable_update_results_table"/>
        </Menu>
        
        <Menu name="search"><text>S&amp;earch</text>
            <Action name="start_search"/>
            <Action name="pause_search"/>
            <Action name="stop_search"/>
            <Separator/>
            <Action name="search_in_background"/>
            <Separator/>
            <Menu name="recheck"><text>Recheck</text>
                <Action name="recheck_broken_items"/>
                <Action name="recheck_visible_items"/>
            </Menu>
            <Separator/>
            <Action name="find_unreferred_documents"/>
        </Menu>

<!--        <Menu name="validate"><text>&amp;Validate</text>
            <Action name="html_fix_all"/>
            <Separator/>
            <Action name="html_fix"/>
            <Separator/>
            <Menu name="manual_validation"><text>Manual Fix</text>
                <Action name="guided_editing"/>
                <Action name="compare_and_apply"/>
            </Menu>
            <Action name="edit_document"/>
        </Menu>-->

        <Menu name="window"><text>&amp;Window</text>
            <Action name="linkcheck_view"/>
            <Action name="unreferred_docs_view"/>
            <Separator/>
            <Menu name="navigation"><text>Navigation</text>
                <Action name="goto_previous_view"/>
                <Action name="goto_next_view"/>
                <Separator/>
                <Action name="goto_previous_session"/>
                <Action name="goto_next_session"/>
            </Menu>
            <Separator/>
            <Action name="goto_check_links"/>
            <Action name="goto_unreferred_documents"/>
        </Menu>

        <Menu name="help"><text>&amp;Help</text>
            <Separator/>
            <Action name="about_klinkstatus"/>
            <Action name="report_bug"/>
        </Menu>
    </MenuBar>
    
    <ToolBar name="linksToolBar">
        <text context="@title:menu">Main Toolbar</text>
        <Action name="start_search"/>
        <Action name="pause_search"/>
        <Action name="stop_search"/>
        <Separator/>
        <Action name="search_in_background"/>
        <Action name="follow_last_link_checked"/>
        <Separator/>
        <Action name="hide_search_bar"/>
        <Action name="reset_search_bar"/>
        <Separator/>
        <Action name="next_view_list"/>
    </ToolBar>
</kpartgui>
