<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="KLinkStatusPluginScripting" library="krossmoduleklinkstatus" version="2">
<MenuBar>
	<Menu name="tools"><text>&amp;Tools</text>
		<Separator/>
		<Action name="edit_script_actions"/>
		<Action name="reset_script_actions"/>
        <Separator/>
	</Menu>
</MenuBar>
</kpartgui>
